import { world, system, ItemStack } from "@minecraft/server"
import { ActionFormData } from "@minecraft/server-ui"


//un objeto que lea el nombre del emote "custom_dance" : "animation.dance.test1"
//entonces solo lee esto y ya estaria
//que se agrupen por paginas y categorias


function formatAnimationName(str) {
    return str
        .split('_')
        .map(word =>
            word.charAt(0).toUpperCase() + word.slice(1).toLowerCase()
        )
        .join(' ');
}

//El nombre de la textura debe ser el mismo del de la animacion
const animationConfig = {
    "penguin_dance": {
        id: "animation.club_penguin",
        loop: true
    },
    "piglin_celebration": {
        id: "animation.piglin.celebrate_hunt_special",
        loop: true
    },
    "doodle_dance": {
        id: "animation.doodle_dance",
        loop: true
    },
    "snow_angel": {
        id: "animation.snow_angel",
        loop: false
    },
    "dab": {
        id: "animation.dab_dance",
        loop: true
    },
    "floss_dance": {
        id: "animation.floss_dance",
        loop: true
    },
    "laugh": {
        id: "animation.laugh",
        loop: true
    },
    "sad": {
        id: "animation.sad",
        loop: false
    },
    "rat_dance": {
        id: "animation.rat_dance",
        loop: true
    },
    "nop": {
        id: "animation.nop",
        loop: false
    },
    "buggie_dance": {
        id: "animation.buggie_dance",
        loop: true
    },
    "jojo_pose": {
        id: "animation.jojo_pose",
        loop: true //esto tambien sirve para animacion con hold on last frame
    },
    "hakari_dance": {
        id: "animation.hakari_dance",
        loop: false
    },
    "griddy": {
        id: "animation.griddy",
        loop: true
    },
    "kazoch_kick": {
        id: "animation.kazoch_kick",
        loop: true
    },
    "cute_dance": {
        id: "animation.cute_dance",
        loop: true
    },
    //Vanila animation
    "bored": {
        id: "animation.react_bored_1",
        loop: false
    },
    "react_bottom": {
        id: "animation.react_bottom_1",
        loop: false
    },

};


function showSelectEmote(player) {
    let actionForm = new ActionFormData().title('Emotes')
        .body('Select an emote: ');

    // Obtenemos un array de las claves
    let keys = Object.keys(animationConfig);

    keys.forEach((clave, index) => {
        let animationName = formatAnimationName(clave);
        actionForm.button(animationName, `textures/emotes/${clave}`);
    });

    actionForm
        .show(player)
        .then(formData => {
            if (formData.canceled) {
                player.playSound('beacon.deactivate');
                return;
            }

            // Accedemos a nuestro objeto de las animaciones
            let selectedKey = keys[formData.selection];
            let animation = animationConfig[selectedKey];

            //Si la animacion va a hacer un loop solo verificamos si se esta moviendo
            const stopExpression = animation.loop ? '(q.is_moving)' : '(q.is_moving || query.all_animations_finished)';
            player.playAnimation(animation.id, { stopExpression: stopExpression });

        });
}



world.beforeEvents.chatSend.subscribe(ev => {
    let message = ev.message
    let player = ev.sender


    if (message == '!e item') {
        ev.cancel = true
        system.run(() => {
            player.dimension.spawnItem(new ItemStack('eu:emote'), player.location)
        })
        return;
    }
    if (message == '!e help') {
        ev.cancel = true
        const keys = Object.keys(animationConfig);
        keys.forEach((clave, index) => {
            player.sendMessage(`${index + 1}: ${clave}`)
        });
        return;
    }
    if (message.startsWith('!e')) {
        ev.cancel = true

        //quitamos '!e ' y nos quedamos con el nombre del animation para buscar su id
        let argument = message.slice(3)
        let animation = animationConfig[argument]

        if (!animation) {
            player.sendMessage(`§cUnknown animation: ${argument}. Please check that the animation exists.`)
            return
        }

        player.sendMessage('Playing animation: ' + `${formatAnimationName(argument)}`)

        const stopExpression = animation.loop ? '(q.is_moving)' : '(q.is_moving || query.all_animations_finished)';
        system.run(() => {
            player.playAnimation(animation.id, { stopExpression: stopExpression })
        })
    }
})


world.afterEvents.itemUse.subscribe(ev => {
    let item = ev.itemStack
    let player = ev.source
    if (item.typeId == 'eu:emote') {
        //player.playAnimation('animation.look', { stopExpression: 'q.is_moving' })

        showSelectEmote(player)
    }
})

